@@ -20,6 +20,9 @@
 
 package Bleach::App;
 use strict;
+use warnings;
+
+use Encode;
 
 sub _hdlr_bleach {
 	my ($ctx, $args, $cond) = @_;
@@ -30,8 +33,15 @@ sub _hdlr_bleach {
 		$text .= MT::Template::Node::nodeValue($t);
 	}
 
+	my $is_utf8 = utf8::is_utf8($text);
+	if ($is_utf8) {
+		$text = encode('utf8', $text);
+	}
 	$text =~ tr/ 	/01/;
-	$text =~ s/[^\r\n]+/pack('b*', $&)/ge;
+	$text =~ s/([^\r\n]+)/pack('b*', $1)/ge;
+	if ($is_utf8) {
+		$text = decode('utf8', $text);
+	}
 
 	$ctx->build($text);
 }
@@ -50,8 +60,15 @@ sub pre_save {
 	my $bleach = sub {
 		my $text = shift;
 		if ($text !~ m/^[\s\r\n]*$/) {
-			$text =~ s/[^\r\n]+/unpack('b*', $&)/ge;
+			my $is_utf8 = utf8::is_utf8($text);
+			if ($is_utf8) {
+				$text = encode('utf-8', $text);
+			}
+			$text =~ s/([^\r\n]+)/unpack('b*', $1)/ge;
 			$text =~ tr/01/ 	/;
+			if ($is_utf8) {
+				$text = decode('utf-8', $text);
+			}
 		}
 		$text;
 	};
@@ -63,4 +80,22 @@ sub pre_save {
 	}
 }
 
+sub source_edit_template {
+	my ($cb, $app, $tmpl) = @_;
+
+	my $replace = '(var editor = CodeMirror)';
+	my $script = '<mt:Var name="bleach_java_script" />';
+	$$tmpl =~ s#$replace#$script\n$1#i;
+}
+
+sub param_edit_template {
+	my ($cb, $app, $param, $tmpl) = @_;
+
+	if ($param->{'text'} =~ m/(<mt:?bleach([^>]*)>)(.*?)(<\/mt:?bleach>)/is) {
+		$param->{'bleach_java_script'} = <<__EOS__;
+parsers.push('../../plugins/Bleach/js/editor.js');
+__EOS__
+	}
+}
+
 1;
